@@ -4,11 +4,12 @@
  *
  * Do ":help uganda"  in Vim to read copying and usage conditions.
  * Do ":help credits" in Vim to see a list of people who contributed.
+ * See README.txt for an overview of the Vim source code.
  */
-
 /*
  * The stuff in this file mostly comes from the "screen" program.
- * Copied from "pty.c" and "putenv.c".
+ * Included with permission from Juergen Weigert.
+ * Copied from "pty.c".  "putenv.c" was used for putenv() in misc2.c.
  *
  * It has been modified to work better with Vim.
  * The parts that are not used in Vim have been deleted.
@@ -36,13 +37,10 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  */
 
-/* RCS_ID("$Id: pty.c,v 1.6 1994/05/31 12:32:44 mlschroe Exp $ FAU") */
+/* RCS_ID("$Id: pty.c,v 1.2 1999/09/19 19:50:51 mool Exp $ FAU") */
 
 #include "vim.h"
 
-#define debug(x)
-#define debug1(x, y)
-
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
@@ -53,7 +51,45 @@
 # include <sys/termios.h>
 #endif
 
-#if !defined(sun) && !defined(VMS) && !defined(macintosh)
+#if HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif
+
+#if HAVE_STROPTS_H
+#include <sys/types.h>
+#ifdef sinix
+#define buf_T __system_buf_t__
+#endif
+#include <stropts.h>
+#ifdef sinix
+#undef buf_T
+#endif
+# ifdef sun
+#  include <sys/conf.h>
+# endif
+#endif
+
+#if HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
+#if HAVE_TERMIO_H
+# include <termio.h>
+#else
+# if HAVE_TERMIOS_H
+#  include <termios.h>
+# endif
+#endif
+
+#if HAVE_SYS_STREAM_H
+# include <sys/stream.h>
+#endif
+
+#if HAVE_SYS_PTEM_H
+# include <sys/ptem.h>
+#endif
+
+#if !defined(sun) && !defined(VMS) && !defined(MACOS)
 # include <sys/ioctl.h>
 #endif
 
@@ -69,7 +105,7 @@
 
 #ifdef sgi
 # include <sys/sysmacros.h>
-#endif /* sgi */
+#endif
 
 #if defined(_INCLUDE_HPUX_SOURCE) && !defined(hpux)
 # define hpux
@@ -85,22 +121,12 @@
 # define PTYRANGE1 "0123456789abcdef"
 #endif
 
-#if !(defined(sequent) || defined(_SEQUENT_) || defined(SVR4))
-# ifdef hpux
-static char PtyProto[] = "/dev/ptym/ptyXY";
-static char TtyProto[] = "/dev/pty/ttyXY";
-# else
-#  ifdef __BEOS__
-static char PtyProto[] = "/dev/pt/XY";
-static char TtyProto[] = "/dev/tt/XY";
-#  else
-static char PtyProto[] = "/dev/ptyXY";
-static char TtyProto[] = "/dev/ttyXY";
-#  endif
-# endif
+/* SVR4 pseudo ttys don't seem to work with SCO-5 */
+#ifdef M_UNIX
+# undef HAVE_SVR4_PTYS
 #endif
 
-static void initpty __ARGS((int));
+static void initmaster __ARGS((int));
 
 /*
  *  Open all ptys with O_NOCTTY, just to be on the safe side
@@ -111,21 +137,49 @@ static void initpty __ARGS((int));
 #endif
 
     static void
-initpty(f)
+initmaster(f)
     int f;
 {
-#ifdef POSIX
+#ifndef VMS
+# ifdef POSIX
     tcflush(f, TCIOFLUSH);
-#else
-# ifdef TIOCFLUSH
-    (void) ioctl(f, TIOCFLUSH, (char *) 0);
+# else
+#  ifdef TIOCFLUSH
+    (void)ioctl(f, TIOCFLUSH, (char *) 0);
+#  endif
+# endif
+# ifdef LOCKPTY
+    (void)ioctl(f, TIOCEXCL, (char *) 0);
 # endif
 #endif
-#ifdef LOCKPTY
-    (void) ioctl(f, TIOCEXCL, (char *) 0);
-#endif
 }
 
+/*
+ * This causes a hang on some systems, but is required for a properly working
+ * pty on others.  Needs to be tuned...
+ */
+    int
+SetupSlavePTY(fd)
+    int fd;
+{
+    if (fd < 0)
+	return 0;
+#if defined(I_PUSH) && defined(HAVE_SVR4_PTYS) && !defined(sgi) && !defined(linux) && !defined(__osf__) && !defined(M_UNIX)
+# if defined(HAVE_SYS_PTEM_H) || defined(hpux)
+    if (ioctl(fd, I_PUSH, "ptem") != 0)
+	return -1;
+# endif
+    if (ioctl(fd, I_PUSH, "ldterm") != 0)
+	return -1;
+# ifdef sun
+    if (ioctl(fd, I_PUSH, "ttcompat") != 0)
+	return -1;
+# endif
+#endif
+    return 0;
+}
+
+
 #if defined(OSX) && !defined(PTY_DONE)
 #define PTY_DONE
     int
@@ -137,13 +191,14 @@ OpenPTY(ttyn)
 
     if ((f = open_controlling_pty(TtyName)) < 0)
 	return -1;
-    initpty(f);
+    initmaster(f);
     *ttyn = TtyName;
     return f;
 }
 #endif
 
-#if (defined(sequent) || defined(_SEQUENT_)) && !defined(PTY_DONE)
+#if (defined(sequent) || defined(_SEQUENT_)) && defined(HAVE_GETPSEUDOTTY) \
+	&& !defined(PTY_DONE)
 #define PTY_DONE
     int
 OpenPTY(ttyn)
@@ -162,7 +217,7 @@ OpenPTY(ttyn)
 #endif
     strncpy(PtyName, m, sizeof(PtyName));
     strncpy(TtyName, s, sizeof(TtyName));
-    initpty(f);
+    initmaster(f);
     *ttyn = TtyName;
     return f;
 }
@@ -188,7 +243,7 @@ OpenPTY(ttyn)
 
     if (name == 0)
 	return -1;
-    initpty(f);
+    initmaster(f);
     *ttyn = name;
     return f;
 }
@@ -203,25 +258,25 @@ OpenPTY(ttyn)
     int		f;
     struct stat buf;
     /* used for opening a new pty-pair: */
-    static char PtyName[32];
     static char TtyName[32];
 
-    strcpy(PtyName, "/dev/ptc");
-    if ((f = open(PtyName, O_RDWR | O_NOCTTY | O_NONBLOCK | O_EXTRA, 0)) < 0)
+    if ((f = open("/dev/ptc", O_RDWR | O_NOCTTY | O_NONBLOCK | O_EXTRA, 0)) < 0)
 	return -1;
-    if (fstat(f, &buf) < 0)
+    if (mch_fstat(f, &buf) < 0)
     {
 	close(f);
 	return -1;
     }
     sprintf(TtyName, "/dev/ttyq%d", minor(buf.st_rdev));
-    initpty(f);
+    initmaster(f);
     *ttyn = TtyName;
     return f;
 }
 #endif
 
-#if defined(SVR4) && !defined(PTY_DONE)
+#if defined(HAVE_SVR4_PTYS) && !defined(PTY_DONE) && !defined(hpux)
+
+/* NOTE: Even though HPUX can have /dev/ptmx, the code below doesn't work! */
 #define PTY_DONE
     int
 OpenPTY(ttyn)
@@ -232,11 +287,9 @@ OpenPTY(ttyn)
     int unlockpt __ARGS((int)), grantpt __ARGS((int));
     RETSIGTYPE (*sigcld)__ARGS(SIGPROTOARG);
     /* used for opening a new pty-pair: */
-    static char PtyName[32];
     static char TtyName[32];
 
-    strcpy(PtyName, "/dev/ptmx");
-    if ((f = open(PtyName, O_RDWR | O_NOCTTY | O_EXTRA, 0)) == -1)
+    if ((f = open("/dev/ptmx", O_RDWR | O_NOCTTY | O_EXTRA, 0)) == -1)
 	return -1;
 
     /*
@@ -252,7 +305,7 @@ OpenPTY(ttyn)
     }
     signal(SIGCHLD, sigcld);
     strncpy(TtyName, m, sizeof(TtyName));
-    initpty(f);
+    initmaster(f);
     *ttyn = TtyName;
     return f;
 }
@@ -271,20 +324,18 @@ OpenPTY(ttyn)
 {
     int		f;
     /* used for opening a new pty-pair: */
-    static char PtyName[32];
     static char TtyName[32];
 
     /* a dumb looking loop replaced by mycrofts code: */
-    strcpy (PtyName, "/dev/ptc");
-    if ((f = open (PtyName, O_RDWR | O_NOCTTY | O_EXTRA)) < 0)
+    if ((f = open("/dev/ptc", O_RDWR | O_NOCTTY | O_EXTRA)) < 0)
 	return -1;
     strncpy(TtyName, ttyname(f), sizeof(TtyName));
-    if (geteuid() && access(TtyName, R_OK | W_OK))
+    if (geteuid() && mch_access(TtyName, R_OK | W_OK))
     {
 	close(f);
 	return -1;
     }
-    initpty(f);
+    initmaster(f);
 # ifdef _IBMR2
     if (aixhack >= 0)
 	close(aixhack);
@@ -300,6 +351,20 @@ OpenPTY(ttyn)
 #endif
 
 #ifndef PTY_DONE
+
+# ifdef hpux
+static char PtyProto[] = "/dev/ptym/ptyXY";
+static char TtyProto[] = "/dev/pty/ttyXY";
+# else
+#  ifdef __BEOS__
+static char PtyProto[] = "/dev/pt/XY";
+static char TtyProto[] = "/dev/tt/XY";
+#  else
+static char PtyProto[] = "/dev/ptyXY";
+static char TtyProto[] = "/dev/ttyXY";
+#  endif
+# endif
+
     int
 OpenPTY(ttyn)
     char **ttyn;
@@ -310,7 +375,6 @@ OpenPTY(ttyn)
     static char PtyName[32];
     static char TtyName[32];
 
-    debug("OpenPTY: Using BSD style ptys.\n");
     strcpy(PtyName, PtyProto);
     strcpy(TtyName, TtyProto);
     for (p = PtyName; *p != 'X'; p++)
@@ -321,17 +385,16 @@ OpenPTY(ttyn)
     {
 	for (d = PTYRANGE1; (p[1] = *d) != '\0'; d++)
 	{
-	    debug1("OpenPTY tries '%s'\n", PtyName);
-#ifdef macintosh
-	    if ((f = open(PtyName, O_RDWR | O_NOCTTY | O_EXTRA)) == -1)
-#else
+#if !defined(MACOS) || defined(USE_CARBONIZED)
 	    if ((f = open(PtyName, O_RDWR | O_NOCTTY | O_EXTRA, 0)) == -1)
+#else
+	    if ((f = open(PtyName, O_RDWR | O_NOCTTY | O_EXTRA)) == -1)
 #endif
 		continue;
 	    q[0] = *l;
 	    q[1] = *d;
-#ifndef macintosh
-	    if (geteuid() && access(TtyName, R_OK | W_OK))
+#ifndef MACOS
+	    if (geteuid() && mch_access(TtyName, R_OK | W_OK))
 	    {
 		close(f);
 		continue;
@@ -352,7 +415,7 @@ OpenPTY(ttyn)
 		}
 	    }
 #endif
-	    initpty(f);
+	    initmaster(f);
 	    *ttyn = TtyName;
 	    return f;
 	}
@@ -360,147 +423,3 @@ OpenPTY(ttyn)
     return -1;
 }
 #endif
-
-/* putenv.c */
-
-/*
- *  putenv  --	put value into environment
- *
- *  Usage:  i = putenv (string)
- *    int i;
- *    char  *string;
- *
- *  where string is of the form <name>=<value>.
- *  Putenv returns 0 normally, -1 on error (not enough core for malloc).
- *
- *  Putenv may need to add a new name into the environment, or to
- *  associate a value longer than the current value with a particular
- *  name.  So, to make life simpler, putenv() copies your entire
- *  environment into the heap (i.e. malloc()) from the stack
- *  (i.e. where it resides when your process is initiated) the first
- *  time you call it.
- *
- *  (history removed, not very interesting.  See the "screen" sources.)
- */
-/* RCS_ID("$Id: putenv.c,v 1.1.1.1 1993/06/16 23:51:15 jnweiger Exp $ FAU") */
-
-#if !defined(HAVE_SETENV) && !defined(HAVE_PUTENV)
-
-#define EXTRASIZE 5		/* increment to add to env. size */
-
-static int  envsize = -1;	/* current size of environment */
-#ifndef macintosh
-extern
-#endif
-       char **environ;		/* the global which is your env. */
-
-static int  findenv();		/* look for a name in the env. */
-static int  newenv();		/* copy env. from stack to heap */
-static int  moreenv();		/* incr. size of env. */
-
-    int
-putenv(string)
-    const char *string;
-{
-    int	    i;
-    char    *p;
-
-    if (envsize < 0)
-    {				/* first time putenv called */
-	if (newenv() < 0)	/* copy env. to heap */
-	    return -1;
-    }
-
-    i = findenv(string);	/* look for name in environment */
-
-    if (i < 0)
-    {				/* name must be added */
-	for (i = 0; environ[i]; i++);
-	if (i >= (envsize - 1))
-	{			/* need new slot */
-	    if (moreenv() < 0)
-		return -1;
-	}
-	p = (char *)alloc((unsigned)(strlen(string) + 1));
-	if (p == NULL)		/* not enough core */
-	    return -1;
-	environ[i + 1] = 0;	/* new end of env. */
-    }
-    else
-    {				/* name already in env. */
-	p = vim_realloc(environ[i], strlen(string) + 1);
-	if (p == NULL)
-	    return -1;
-    }
-    sprintf(p, "%s", string);	/* copy into env. */
-    environ[i] = p;
-
-    return 0;
-}
-
-    static int
-findenv(name)
-    char *name;
-{
-    char    *namechar, *envchar;
-    int	    i, found;
-
-    found = 0;
-    for (i = 0; environ[i] && !found; i++)
-    {
-	envchar = environ[i];
-	namechar = name;
-	while (*namechar && *namechar != '=' && (*namechar == *envchar))
-	{
-	    namechar++;
-	    envchar++;
-	}
-	found = ((*namechar == '\0' || *namechar == '=') && *envchar == '=');
-    }
-    return found ? i - 1 : -1;
-}
-
-    static int
-newenv()
-{
-    char    **env, *elem;
-    int	    i, esize;
-
-    for (i = 0; environ[i]; i++)
-	;
-    esize = i + EXTRASIZE + 1;
-    env = (char **)alloc((unsigned)(esize * sizeof (elem)));
-    if (env == NULL)
-	return -1;
-
-    for (i = 0; environ[i]; i++)
-    {
-	elem = (char *)alloc((unsigned)(strlen(environ[i]) + 1));
-	if (elem == NULL)
-	    return -1;
-	env[i] = elem;
-	strcpy(elem, environ[i]);
-    }
-
-    env[i] = 0;
-    environ = env;
-    envsize = esize;
-    return 0;
-}
-
-    static int
-moreenv()
-{
-    int	    esize;
-    char    **env;
-
-    esize = envsize + EXTRASIZE;
-    env = (char **)vim_realloc((char *)environ, esize * sizeof (*env));
-    if (env == 0)
-	return -1;
-    environ = env;
-    envsize = esize;
-    return 0;
-}
-
-#endif /* !defined(HAVE_SETENV) && !defined(HAVE_PUTENV) */
